local _, addon = ...

-- functions
local C_Timer = C_Timer
local C_Timer_NewTimer = C_Timer.NewTimer
local CreateFrame = CreateFrame
local GetContainerItemID = GetContainerItemID
local GetContainerItemLink = GetContainerItemLink
local GetContainerNumSlots = GetContainerNumSlots
-- constants
local NUM_BAG_SLOTS = NUM_BAG_SLOTS
-- frames
-- libs

addon:RegisterCallback(addon.EVENT_BAG_SCAN_STARTED, function()
    addon:Print("Scanning for keystones in bags...")
end)

addon:RegisterCallback(addon.EVENT_BAG_SCAN_FINISHED, function(_, bagId, slotId, link)
    local player = addon.player

    if not bagId then
        addon:Print("Could not find keystone in bags.")
        addon:RemoveCharacter("player", {player.realm, player.faction, player.name})
        return
    end

    addon:Print(string.format("Found `%s`.", link))
    local _, instanceMapId, instanceName, level, isDepleted = addon:GetKeystoneInformation(link)
    addon:AddCharacter(
        "player",
        {
            player.realm,
            player.faction,
            player.name,
            player.class,
            {player.guild.realm, player.faction, player.guild.name,},
            {{link, instanceMapId, instanceName, level, isDepleted, bagId, slotId}},
            {}
        })
end)

do
    local scanDelay = 3
    local scanTimer
    local function scanCallback()
        local foundBagId, foundSlotId, foundLink

        addon:Fire(addon.EVENT_BAG_SCAN_STARTED)

        for bagId = 0, NUM_BAG_SLOTS do
            for slotId = 0, GetContainerNumSlots(bagId) do
                local itemId = GetContainerItemID(bagId, slotId)
                if 138019 == itemId then
                    foundBagId, foundSlotId = bagId, slotId
                    foundLink = GetContainerItemLink(bagId, slotId)
                end
            end
        end

        addon:Fire(addon.EVENT_BAG_SCAN_FINISHED, foundBagId, foundSlotId, foundLink)
    end

    local function onEvent()
        if scanTimer then
            scanTimer:Cancel()
        end

        scanTimer = C_Timer_NewTimer(scanDelay, scanCallback)
    end

    local f = CreateFrame("frame")
    f:SetScript("OnEvent", onEvent)
    f:RegisterEvent("BAG_UPDATE_DELAYED")
    f:RegisterEvent("CHALLENGE_MODE_COMPLETED")
    f:RegisterEvent("CHALLENGE_MODE_RESET")
    f:RegisterEvent("CHALLENGE_MODE_START")
    f:RegisterEvent("PLAYER_ENTERING_WORLD")

    function addon:BagScannerOnEvent()
        onEvent()
    end
end
