local _, addon = ...

-- functions
local unpack = unpack
local format = string.format
-- constants
local RAID_CLASS_COLORS = RAID_CLASS_COLORS
-- frames
-- libs

--[[
character {
    realm,
    faction,
    class,
    guild {
        realm,
        faction,
        name
    },
    keystones {
        {instanceMapId, instanceName, level, isDepleted, bagId, slotId},
    },
    records: {
    },
}
]]
function addon:AddCharacter(pool, character)
    if not self.db.pools[pool] then self.db.pools[pool] = {} end

    local realm, faction, name, class, guild, keystones, records = unpack(character)
    local key = format("%s.%s.%s", realm, faction, name)

    local changed = false
    if self.db.pools[pool][key] ~= nil then
        changed = true
        self:Print(format("Updating character `%s` in pool `%s`.", key, pool))
    else
        self:Print(format("Adding character `%s` to pool `%s`.", key, pool))
    end

    self.db.pools[pool][key] = character

    self:Fire(changed and self.EVENT_CHARACTER_CHANGED or self.EVENT_CHARACTER_ADDED, pool, character)
end

function addon:RemoveCharacter(pool, character)
    if not self.db.pools[pool] then self.db.pools[pool] = {} end

    local realm, faction, name = unpack(character)
    local key = format("%s.%s.%s", realm, faction, name)

    if self.db.pools[pool][key] ~= nil then
        self:Print(format("Removing character `%s` from pool `%s`.", key, pool))
    end

    self.db.pools[pool][key] = nil
    self:Fire(self.EVENT_CHARACTER_REMOVED, pool, character)
end

function addon:CharacterIsMe(character)
    local realm, faction, name, class, guild, keystones, records = unpack(character)

    return realm == self.player.realm and faction == self.player.faction and name == self.player.name
end

function addon:GetCharacterName(colour, ...)
    local realm, _, name, class = ...
    if realm ~= self.player.realm then
        if colour then
            return format("|c%s%s-%s|r", RAID_CLASS_COLORS[class].colorStr, name, realm)
        else
            return format("%s-%s", name, realm)
        end
    end

    if colour then
        return format("|c%s%s|r", RAID_CLASS_COLORS[class].colorStr, name)
    end

    return name
end

function addon:GetFactionName(colour, faction)
    if colour then
        local hex = "FF0000"
        if faction == "Alliance" then
            hex = "2459FF"
        end

        return format("|cff%s%s|r", hex, faction)
    end

    return faction
end

function addon:GetCharacterDisplayName(colour, ...)
    local faction = select(2, ...)
    if faction ~= self.player.faction then
        return format("%s (%s)", self:GetCharacterName(colour, ...), self:GetFactionName(colour, faction))
    end

    return self:GetCharacterName(colour, ...)
end
