local _, addon = ...

-- functions
local format = string.format
local CreateFrame = CreateFrame
local GetGuildInfo = GetGuildInfo
local GetRealmName = GetRealmName
local GuildRoster = GuildRoster
local UnitClass = UnitClass
local UnitFactionGroup = UnitFactionGroup
local UnitName = UnitName
-- constants
-- frames
-- libs

addon:RegisterCallback(addon.EVENT_INITIALISED, function()
    addon:Print("Ready.")
end)

addon:RegisterCallback(addon.EVENT_GUILD_INFO_CHANGED, function(_, guildName, guildRealm)
    addon.player.guild.name = guildName
    addon.player.guild.realm = guildRealm or addon.player.realm

    addon:Print(format("New guild info: `%s.%s.%s`", addon.player.guild.realm, addon.player.faction, addon.player.guild.name))
end)

function addon:Initialise(db)
    self:Print("Initialising.")

    local unit = "player"

    self.db = db
    self.debug = self.db.debug and self.db.debug == true

    self.db.pools = self.db.pools or {}
    self.db.pools[unit] = self.db.pools[unit] or {}

    local name = select(1, UnitName(unit))
    self.player = {
        name = name or "",
        realm = GetRealmName() or "",
        class = select(2, UnitClass(unit)) or "",
        faction = select(2, UnitFactionGroup(unit)) or "",
        guild = {
            name = nil,
            realm = nil,
        },
    }

    -- listen for guild changes
    do
        local f = CreateFrame("frame")
        f:SetScript("OnEvent", function()
            local guildName, guildRank, _, guildRealm = GetGuildInfo(unit)
            if not guildName or #guildName < 1 then return end

            self:Fire(self.EVENT_GUILD_INFO_CHANGED, guildName, guildRealm)
        end)
        f:RegisterEvent("GUILD_MOTD")
        --f:RegisterEvent("GUILD_ROSTER_UPDATE")
        f:RegisterEvent("PLAYER_GUILD_UPDATE")
        GuildRoster()
    end

    self:Fire(self.EVENT_INITIALISED)
end

function addon:Print(...)
    if not self.debug or not ... then return end

    local obj
    for i = 1, #... do
        obj = select(i, ...)
        if obj and type(obj) == "table" then
            print(format("|cff%sBK:|r", "6699CC"))
            DevTools_Dump(obj)
        elseif obj then
            print(format("|cff%sBK:|r %s", "6699CC", type(obj) == "string" and format(obj, ...) or tostring(obj)))
        end
    end
end
